module Ubazu
  module Rspec # :nodoc:
    module Matchers
      module ActiveRecord # :nodoc:

        # Ensures that the model field value is included in specified array.
        #
        # Options:
        # * <tt>with_message</tt> - value the test expects to find in
        #   <tt>errors.on(:attribute)</tt>. <tt>Regexp</tt> or <tt>String</tt>.
        #   Defaults to the translation for <tt>:blank</tt>.
        #
        # Examples:
        #   it { should validate_inclusion_of(:name).in(:array) }
        #   it { should validate_inclusion_of(:name).in(:array)
        #          with_message(/is not optional/) }
        #
        def validate_inclusion_of(attr)
          ValidateInclusionOfMatcher.new(attr)
        end

        class ValidateInclusionOfMatcher < ValidationMatcher # :nodoc:

          def in(array)
            @array = array
            self
          end

          def with_message(message)
            @expected_message = message if message
            self
          end

          def matches?(instance)
            @instance = instance
            super(instance)

            @expected_message ||= :blank
            wrong_value = rand(99999999999).to_s.to_sym #wrong symbol
            @instance.send("#{@attribute}=", wrong_value)
            result = !errors_match?

            if result
              true
            else
              @failure_message = "#{@attribute} is not validated for inclusion in #{@array.inspect}"
              false
            end
          end

          def description
            "ensured inclusion of #{@attribute} in #{@array.inspect}"
          end

          private

          def errors_match?
            @instance.valid?
            @errors = errors_for_attribute(@instance, @attribute)
            @errors = [@errors] unless @errors.is_a?(Array)
            @expected_message ? (errors_match_regexp? || errors_match_string?) : (@errors.compact.any?)
          end

          def errors_match_regexp?
            if Regexp === @expected_message
              @matched_error = @errors.detect { |e| e =~ @expected_message }
              !@matched_error.nil?
            else
              false
            end
          end

          def errors_match_string?
            if @errors.include?(@expected_message)
              @matched_error = @expected_message
              true
            else
              false
            end
          end

          def blank_value
            if collection?
              []
            else
              nil
            end
          end

        end
      end
    end
  end
end
